<?php

namespace WPT\Recipe\Recipe;

use  Carbon_Fields\Field ;
use  Carbon_Fields\Container ;
/**
 * RecipePostMeta.
 */
class PostMetaFields
{
    protected  $container ;
    /**
     * Constructor.
     */
    public function __construct( $container )
    {
        $this->container = $container;
    }
    
    /**
     * Register Post Meta Fields.
     */
    public function register_fields()
    {
        $container = Container::make( 'post_meta', 'Recipe Information' )->set_context( 'advanced' )->where( 'post_type', '=', 'wpt-food-recipe' )->add_fields( $this->get_fields() );
        return $container;
    }
    
    /**
     * post meta fields
     */
    public function get_fields()
    {
        $fields = [];
        $fields[] = Field::make( 'html', 'wpt_recipe_media_section', 'Section Description' )->set_html( '<h4 style="color:#007cba">Media Section</h4><i>Information regarding media related to the recipe</i>' );
        $fields[] = Field::make( 'rich_text', 'wpt_recipe_preamble', 'Preamble' )->set_help_text( 'Preamble for the recipe. Its not part of the recipe SEO. Can be used in layouts which have text before the actual recipe.' );
        $fields[] = Field::make( 'image', 'wpt_recipe_image', 'Image' )->set_help_text( 'Image of the completed dish. For best results, provide high-resolution image (minimum of 50K pixels when multiplying width and height) with the following aspect ratios: 16x9, 4x3, and 1x1.' );
        $fields[] = Field::make( 'html', 'wpt_recipe_general_section', '' )->set_html( '<h4 style="color:#007cba">General Section</h4>' );
        $fields[] = Field::make( 'textarea', 'wpt_recipe_description', 'Description' )->set_help_text( 'A short summary describing the dish.' );
        $fields[] = Field::make( 'select', 'wpt_recipe_author', 'Author' )->set_options( [
            'none'        => 'None',
            'post_author' => 'Post Author',
            'custom'      => 'Custom',
        ] );
        $fields[] = Field::make( 'text', 'wpt_recipe_author_name', 'Author Name' )->set_help_text( 'Enter name of the author' )->set_conditional_logic( [ [
            'field' => 'wpt_recipe_author',
            'value' => 'custom',
        ] ] );
        $fields[] = Field::make( 'text', 'wpt_recipe_author_url', 'Author URL' )->set_help_text( 'URL representing the author' )->set_conditional_logic( [ [
            'field' => 'wpt_recipe_author',
            'value' => 'custom',
        ] ] );
        $fields[] = Field::make( 'text', 'wpt_recipe_author_img', 'Author Image' )->set_help_text( 'Author Image URL' )->set_conditional_logic( [ [
            'field' => 'wpt_recipe_author',
            'value' => 'custom',
        ] ] );
        $fields[] = Field::make( 'text', 'wpt_recipe_yield', 'Serving/Yield' )->set_attribute( 'type', 'number' )->set_attribute( 'min', '1' )->set_help_text( 'The quantity produced by the recipe. Specify the number of servings produced from this recipe with just a number' );
        $fields[] = Field::make( 'text', 'wpt_recipe_yield_unit', 'Serving/Yield Unit' )->set_attribute( 'placeholder', 'people/cookies' )->set_help_text( 'Unit of the serving. For example people, cups, cookies etc' );
        $fields[] = Field::make( 'text', 'wpt_recipe_cost', 'Estimated Cost' )->set_attribute( 'placeholder', '$9' )->set_help_text( 'Estimated cost of the materials/effort needed to create the recipe' );
        $fields[] = Field::make( 'html', 'wpt_recipe_categories_section', '' )->set_html( '<h4 style="color:#007cba">Recipe Categories Section</h4>' );
        $fields[] = Field::make( 'set', 'wpt_recipe_categories', 'Categories' )->add_options( [
            'Breakfast'   => 'Breakfast',
            'Appetizer'   => 'Appetizer',
            'Soup'        => 'Soup',
            'Main Course' => 'Main Course',
            'Side Dish'   => 'Side Dish',
            'Salad'       => 'Salad',
            'Dessert'     => 'Dessert',
            'Snack'       => 'Snack',
            'Drinks'      => 'Drinks',
        ] )->set_help_text( 'The type of meal or course your recipe is about. For example: "dinner", "main course", or "dessert, snack".' );
        $fields[] = Field::make( 'text', 'wpt_recipe_categories_others', '' )->set_attribute( 'placeholder', 'For any other categories, enter a comma-separated list over here' )->set_help_text( '' );
        $fields[] = Field::make( 'set', 'wpt_recipe_cuisines', 'Cuisines' )->add_options( [
            'French'        => 'French',
            'Italian'       => 'Italian',
            'Mediterranean' => 'Mediterranean',
            'Indian'        => 'Indian',
            'Chinese'       => 'Chinese',
            'Japanese'      => 'Japanese',
            'American'      => 'American',
            'Mexican'       => 'Mexican',
        ] )->set_help_text( 'The region associated with your recipe. For example, "French", Mediterranean", or "American".' );
        $fields[] = Field::make( 'text', 'wpt_recipe_cuisines_others', '' )->set_attribute( 'placeholder', 'For any other cuisine(s), enter a comma-separated list over here' )->set_help_text( '' );
        $fields[] = Field::make( 'text', 'wpt_recipe_keywords', 'Keywords' )->set_attribute( 'placeholder', 'winter apple pie, nutmeg crust' )->set_help_text( 'Other terms for your recipe such as the season ("summer"), the holiday ("Halloween"), or other descriptors ("quick", "easy", "authentic"). Comma-separated values' );
        $fields[] = Field::make( 'html', 'wpt_recipe_ingredients_section', '' )->set_html( '<h4 style="color:#007cba">Ingredients / Materials Section</h4>' );
        $this->set_ingredient_field( $fields );
        $fields[] = Field::make( 'html', 'wpt_recipe_instructions_section', '' )->set_html( '<h4 style="color:#007cba">Instructions Section</h4>' );
        $this->set_instructions_fields( $fields );
        $fields[] = Field::make( 'html', 'wpt_recipe_time_section', '' )->set_html( '<h4 style="color:#007cba">Time Section</h4>' );
        $fields[] = Field::make( 'iso8601duration', 'wpt_recipe_prep_time', 'Prep Time' )->set_help_text( 'The length of time it takes to prepare the dish' );
        $fields[] = Field::make( 'iso8601duration', 'wpt_recipe_cook_time', 'Cook Time' )->set_help_text( 'The time it takes to actually cook the dish' );
        $fields[] = Field::make( 'rich_text', 'wpt_recipe_notes', 'Notes' )->set_help_text( 'Additional notes.' );
        return $fields;
    }
    
    /**
     * Instructions field
     */
    public function set_instructions_fields( &$fields )
    {
        $fields[] = Field::make( 'radio', 'wpt_recipe_instruction_type', 'Instruction Type' )->set_options( [
            'simple'  => 'Simple List (Recommended)',
            'grouped' => 'Grouped List',
        ] )->set_help_text( 'Select the type of instruction for the recipe. Set it to "Simple List" for a simple list of instructions. Select "Grouped List" to create a group of instructions (For example recipe For pizza containing group instructions for "Red Sauce", "Pizza Dough" and "Making Pizza"). ' );
        $simple_instruction_field = $this->get_instruction_steps_field( 'wp_recipe_simple_instructions' );
        $simple_instruction_field->set_conditional_logic( [ [
            'field'   => 'wpt_recipe_instruction_type',
            'value'   => 'simple',
            'compare' => '=',
        ] ] );
        $fields[] = $simple_instruction_field;
        $fields[] = Field::make( 'complex', 'wpt_recipe_grouped_instructions', '' )->set_help_text( 'Group of steps to make the dish.' )->setup_labels( [
            'plural_name'   => 'Instruction Groups',
            'singular_name' => 'Instruction Group',
        ] )->add_fields( [ Field::make( 'text', 'name', 'Group Name' )->set_help_text( 'Name of the group section' ), $this->get_instruction_steps_field( 'steps' ) ] )->set_conditional_logic( [ [
            'field'   => 'wpt_recipe_instruction_type',
            'value'   => 'grouped',
            'compare' => '=',
        ] ] )->set_header_template( '<%- name %>' );
    }
    
    public function get_instruction_steps_field( $field_name )
    {
        $media_options = [
            'none' => 'None',
        ];
        $fields = [];
        $fields[] = Field::make( 'text', 'name', 'Name' )->set_classes( 'one_four' )->set_help_text( 'Title of the step' );
        $fields[] = Field::make( 'textarea', 'instruction', 'Instruction Text' )->set_classes( 'one_four' )->set_help_text( 'Step text instruction.' );
        $fields[] = Field::make( 'select', 'media_type', 'Media Type' )->set_classes( 'one_four' )->set_help_text( 'Select Image or Video if you want to associate an image/video with this step. Else select none.' )->set_options( $media_options );
        $fields[] = Field::make( 'image', 'step_image', 'Image' )->set_classes( 'one_four' )->set_help_text( 'An image representing this step.' )->set_conditional_logic( [ [
            'field'   => 'media_type',
            'value'   => 'image',
            'compare' => '=',
        ] ] );
        return Field::make( 'complex', $field_name, 'Steps' )->set_help_text( 'Steps of instructions.' )->setup_labels( [
            'plural_name'   => 'Steps',
            'singular_name' => 'Step',
        ] )->add_fields( $fields )->set_header_template( '
                    <%- name %>
                    ' );
    }
    
    public function set_ingredient_field( &$fields )
    {
        $ingredient_field = Field::make( 'complex', 'ingredients', '' )->add_fields( [
            Field::make( 'text', 'quantity', 'Quantity' )->set_classes( 'one_five' )->set_help_text( 'The quantity of the ingredient.' ),
            Field::make( 'association', 'ingredient', 'Name' )->set_classes( 'two_five' )->set_types( [ [
            'type'      => 'post',
            'post_type' => 'wpt-food-ingredients',
        ] ] )->set_min( 1 )->set_max( 1 )->set_help_text( 'Select the ingredient.' ),
            Field::make( 'text', 'unit', 'Quantity Unit' )->set_classes( 'one_five' )->set_help_text( 'Add the unit for the quantity' ),
            Field::make( 'text', 'notes', 'Notes' )->set_classes( 'one_five' )->set_help_text( 'Any notes related to the ingredient.' )
        ] )->setup_labels( [
            'plural_name'   => 'Ingredients',
            'singular_name' => 'Ingredient',
        ] )->set_help_text( 'Add the ingredient information related to the recipe' );
        $fields[] = Field::make( 'complex', 'wpt_recipe_ingredients', '' )->set_help_text( 'Create an ingredient group and add ingredients to that group. For example create a "Pizza Toppings" group and add the pizza toppings like mushrooms, onions, tomatoes etc.' )->add_fields( [ Field::make( 'text', 'group_name', 'Group Name' )->set_help_text( 'Add the group name for the ingredient. For example Pizza Dough, Pizza Topping etc' ), $ingredient_field ] )->setup_labels( [
            'plural_name'   => 'Ingredient Groups',
            'singular_name' => 'Ingredient Group',
        ] )->set_header_template( '
                <%- group_name %>
                ' );
    }

}